#include "Beam.h"


const float Beam::massPerLength = 0.01f, Beam::kStiffness = 1000.0f, Beam::forceBeamMax = 2000.0f;




Beam::Beam(SDL_Renderer* renderer, std::shared_ptr<Pin>& setPinConnection1,
	std::shared_ptr<Pin>& setPinConnection2, bool setIsRoad) :
	pinConnection1(setPinConnection1), pinConnection2(setPinConnection2),
	isRoad(setIsRoad) {
	//Setup the initial length of the beam and it's mass.
	if (pinConnection1 != nullptr && pinConnection2 != nullptr) {
		length0 = (pinConnection1->getPos() - pinConnection2->getPos()).magnitude();
		mass = massPerLength * length0;
	}

	texture = TextureLoader::loadTexture(renderer, (setIsRoad ? "Beam Road.bmp" : "Beam Wood.bmp"));
}



void Beam::draw(SDL_Renderer* renderer) {
	if (pinConnection1 != nullptr && pinConnection2 != nullptr && texture != nullptr) {
		//Set the draw color based on the force in the beam.
		int color = (int)round(255 * (1.0f - colorForceFactor));
		SDL_SetTextureColorMod(texture, 255, color, color);

		//Calculate the position and dimensions that the image will be drawn at.
		Vector2D posCenter = (pinConnection1->getPos() + pinConnection2->getPos()) / 2.0f;
		Vector2D lengthCurrent = pinConnection1->getPos() - pinConnection2->getPos();
		float angleDeg = MathAddon::angleRadToDeg(lengthCurrent.angle());

		//Draw the beam image at the correct position, size, and angle.
		SDL_Rect rect = {
			(int)round(posCenter.x - lengthCurrent.magnitude() / 2.0f),
			(int)round(posCenter.y - thickness / 2.0f),
			(int)round(lengthCurrent.magnitude()),
			thickness };
		SDL_RenderCopyEx(renderer, texture, NULL, &rect, angleDeg, NULL, SDL_FLIP_NONE);
	}
}



bool Beam::getIsBroken() {
	return isBroken;
}



bool Beam::checkIfconnectedToPin(Pin& pin) {
	return ((pinConnection1 != nullptr && pinConnection1.get() == &pin) ||
		(pinConnection2 != nullptr && pinConnection2.get() == &pin));
}



void Beam::addForceAndMassDiv2AtPin(Pin& pin, Vector2D& forceSum, float& massSum) {
	forceSum += getForceAtPin(pin);
	massSum += mass / 2.0f;
}



bool Beam::checkOverlapWithUnit(Vector2D posUnit, float radiusUnit) {
	if (isRoad && pinConnection1 != nullptr && pinConnection2 != nullptr)
		return MathAddon::checkOverlapLineCircle(pinConnection1->getPos(), pinConnection2->getPos(),
			posUnit, (radiusUnit + thickness / 2.0f));

	return false;
}



Vector2D Beam::getForceAtPin(Pin& pin) {
	if (pinConnection1 != nullptr && pinConnection2 != nullptr && length0 > 0.0f) {
		//Determine the current length and displacement of the beam.
		Vector2D lengthCurrent = pinConnection1->getPos() - pinConnection2->getPos();
		Vector2D displacement = Vector2D(lengthCurrent).normalize() * (lengthCurrent.magnitude() - length0);

		//Determine the forces.
		Vector2D forceBeam = displacement * kStiffness;
		Vector2D forceGravity = Vector2D(0.0f, mass * MathAddon::accGravity);

		//Determine how red the beam should be drawn and check if the forces are high enough to break it.
		if (forceBeamMax > 0.0f) {
			colorForceFactor = forceBeam.magnitude() / forceBeamMax;
			if (colorForceFactor >= 1.0f) {
				colorForceFactor = 1.0f;
				isBroken = true;
			}
		}
		
		//Output the forces.
		if (pinConnection1.get() == &pin)
			return (forceBeam * -1.0f) + forceGravity / 2.0f;
		else if (pinConnection2.get() == &pin)
			return forceBeam + forceGravity / 2.0f;
	}

	return Vector2D();
}