#include "Game.h"



Game::Game(SDL_Window* window, SDL_Renderer* renderer, int windowWidth, int windowHeight) :
    gameModeCurrent(GameMode::paused), unitRedCreature(renderer, Vector2D(64.0f, 344.0f)) {

    //Run the game.
    if (window != nullptr && renderer != nullptr) {
        //Load the some important textures.
        textureBackground = TextureLoader::loadTexture(renderer, "Background.bmp");
        textureOverlayInstructions = TextureLoader::loadTexture(renderer, "Overlay Instructions.bmp");
        textureOverlayVictory = TextureLoader::loadTexture(renderer, "Overlay Won.bmp");
        textureOverlayDefeat = TextureLoader::loadTexture(renderer, "Overlay Lost.bmp");

        //Setup a default bridge.
        setupBridge1(renderer);


        //Store the current times for the clock.
        auto time1 = std::chrono::system_clock::now();
        auto time2 = std::chrono::system_clock::now();

        //The amount of time for each frame (60 fps).
        const float dT = 1.0f / 60.0f;


        //Start the game loop and run until it's time to stop.
        bool running = true;
        while (running) {
            //Determine how much time has elapsed since the last frame.
            time2 = std::chrono::system_clock::now();
            std::chrono::duration<float> timeDelta = time2 - time1;
            float timeDeltaFloat = timeDelta.count();

            //If enough time has passed then do everything required to generate the next frame.
            if (timeDeltaFloat >= dT) {
                //Store the new time for the next frame.
                time1 = time2;

                processEvents(renderer, running);
                update(dT);
                draw(renderer);
            }
        }
    }
}


Game::~Game() {
    //Clean up.
    TextureLoader::deallocateTextures();
}



void Game::processEvents(SDL_Renderer* renderer, bool& running) {
    //Process events.
    SDL_Event event;
    while (SDL_PollEvent(&event)) {
        switch (event.type) {
        case SDL_QUIT:
            running = false;
            break;

        case SDL_KEYDOWN:
            switch (event.key.keysym.scancode) {
                //Quit the game.
            case SDL_SCANCODE_ESCAPE:
                running = false;
                break;
            }

            if (gameModeCurrent == GameMode::playing || gameModeCurrent == GameMode::paused) {
                switch (event.key.keysym.scancode) {
                    //Set the current bridge type
                case SDL_SCANCODE_1:
                    setupBridge1(renderer);
                    break;
                case SDL_SCANCODE_2:
                    setupBridge2(renderer);
                    break;

                    //Run/Pause the game
                case SDL_SCANCODE_R:
                    gameModeCurrent = (gameModeCurrent == GameMode::playing ? GameMode::paused : GameMode::playing);
                    break;

                    //Show/hide the overlay
                case SDL_SCANCODE_H:
                    overlayInstructionsVisible = !overlayInstructionsVisible;
                    break;
                }
            }
        }
    }
}



void Game::update(float dT) {
    if (gameModeCurrent == GameMode::playing) {
        //Update the red creature and check if it's stuck or has reached the end of the level.
        unitRedCreature.update(dT, listBeams);
        if (unitRedCreature.getIsStuck())
            gameModeCurrent = GameMode::defeat;
        else if (unitRedCreature.getReachedEndOfLevel())
            gameModeCurrent = GameMode::victory;


        //Calculate the forces at the pins (and in the beams).
        for (auto& pinSelected : listPins)
            if (pinSelected != nullptr)
                pinSelected->calculateForces(listBeams, unitRedCreature);

        //Update the beams.
        for (int count = 0; count < listBeams.size(); count++) {
            auto& beamSelected = listBeams[count];
            if (beamSelected != nullptr && beamSelected->getIsBroken()) {
                listBeams.erase(listBeams.begin() + count);
                count--;
            }
        }

        //Update the pins.
        for (auto pinSelected : listPins)
            if (pinSelected != nullptr)
                pinSelected->update(dT);
    }
}



void Game::draw(SDL_Renderer* renderer) {
    //The unit is alive so set the draw color to dark purple.
    SDL_SetRenderDrawColor(renderer, 118, 175, 255, 255);
    //Clear the screen.
    SDL_RenderClear(renderer);

    //Draw everything here.
    if (textureBackground != nullptr)
        SDL_RenderCopy(renderer, textureBackground, NULL, NULL);

    //Draw the red creature
    unitRedCreature.draw(renderer);

    //Draw the beams.
    for (auto& beamSelected : listBeams)
        if (beamSelected != nullptr)
            beamSelected->draw(renderer);

    //Draw the pins.
    for (auto& pinSelected : listPins)
        if (pinSelected != nullptr)
            pinSelected->draw(renderer);



    switch (gameModeCurrent) {
    case GameMode::playing:
    case GameMode::paused:
        //Draw the instructions overlay.
        if (textureOverlayInstructions != nullptr && overlayInstructionsVisible) {
            int w = 0, h = 0;
            SDL_QueryTexture(textureOverlayInstructions, NULL, NULL, &w, &h);
            SDL_Rect rect = { 40, 40, w, h };
            SDL_RenderCopy(renderer, textureOverlayInstructions, NULL, &rect);
        }
        break;


    case GameMode::victory:
        if (textureOverlayVictory != nullptr)
            SDL_RenderCopy(renderer, textureOverlayVictory, NULL, NULL);
        break;
    case GameMode::defeat:
        if (textureOverlayDefeat != nullptr)
            SDL_RenderCopy(renderer, textureOverlayDefeat, NULL, NULL);
        break;
    }


    //Send the image to the window.
    SDL_RenderPresent(renderer);
}



void Game::setupBridge1(SDL_Renderer* renderer) {
    unitRedCreature.reset();

    listPins.clear();
    listBeams.clear();

    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(264.0f, 387.0f), true));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(372.0f, 387.0f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(480.0f, 387.0f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(588.0f, 387.0f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(696.0f, 387.0f), true));

    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(372.0f, 293.5f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(480.0f, 293.5f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(588.0f, 293.5f)));

    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[0], listPins[1], true));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[1], listPins[2], true));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[2], listPins[3], true));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[3], listPins[4], true));

    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[5], listPins[0]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[5], listPins[1]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[5], listPins[6]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[6], listPins[2]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[7], listPins[3]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[7], listPins[4]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[7], listPins[6]));
}


void Game::setupBridge2(SDL_Renderer* renderer) {
    unitRedCreature.reset();

    listPins.clear();
    listBeams.clear();

    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(264.0f, 387.0f), true));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(372.0f, 387.0f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(480.0f, 387.0f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(588.0f, 387.0f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(696.0f, 387.0f), true));

    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(372.0f, 293.5f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(480.0f, 293.5f)));
    listPins.push_back(std::make_shared<Pin>(renderer, Vector2D(588.0f, 293.5f)));

    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[0], listPins[1], true));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[1], listPins[2], true));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[2], listPins[3], true));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[3], listPins[4], true));

    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[5], listPins[0]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[5], listPins[1]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[5], listPins[6]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[6], listPins[1]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[6], listPins[2]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[6], listPins[3]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[7], listPins[3]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[7], listPins[4]));
    listBeams.push_back(std::make_shared<Beam>(renderer, listPins[7], listPins[6]));
}