#include "Unit.h"


const float Unit::speedX = 100.0f, Unit::size = 64.0f, Unit::mass = 14.5f;




Unit::Unit(SDL_Renderer* renderer, Vector2D setPos) :
	pos(setPos), posStart(setPos), texture(TextureLoader::loadTexture(renderer, "Red Creature.bmp")) {

}



void Unit::update(float dT, std::vector<std::shared_ptr<Beam>>& listBeams) {
	//Calculate the acceleration, update the speed, and determine the desired position
	//if nothing is blocking this unit.
	
	//Note that the speed is split into x and y components because speedX is fixed.
	speedY += MathAddon::accGravity * dT;
	Vector2D posDesired = pos + Vector2D(speedX * dT, speedY * dT);

	//If the unit was at posDesired, check if it would overlap anything.  If it does
	//then adjust it slightly to be closer to the top of the screen and check it again.
	//Keep iterating until it doesn't overlap anything or a maximum number of iterations 
	//(distance) is reached.
	bool posDesiredOk = false;
	for (int count = 0; count < 30 && posDesiredOk == false; count++) {
		if (checkOverlapWithBridgeAndLand(posDesired, listBeams, (count == 0)) == false)
			posDesiredOk = true;
		else {
			posDesired.y -= 0.1f;
			//Reset it's speed in the y direction because it's overlapping something that stopped it.
			speedY = 0.0f;
		}
	}

	//Set the desired position to the current position if it's ok.
	if (posDesiredOk)
		pos = posDesired;

	//Check if it reached the end of the level
	if (pos.x > (960.0f - 64.0f) && pos.y < 376.0f)
		reachedEndOfLevel = true;

	//Check if it got stuck.  Either because it wasn't able to move or is below the screen.
	if (posDesiredOk == false || pos.y > (540.0f + size / 2.0f))
		isStuck = true;
}



void Unit::draw(SDL_Renderer* renderer) {
	if (renderer != nullptr && texture != nullptr) {
		//Draw the image at the unit's position.
		SDL_Rect rect = {
			(int)round(pos.x - size / 2.0f),
			(int)round(pos.y - size / 2.0f),
			(int)round(size),
			(int)round(size) };
		SDL_RenderCopy(renderer, texture, NULL, &rect);
	}
}



void Unit::reset() {
	//Reset the unit to it's state at the beginning of the game.
	pos = posStart;
	speedY = 0.0f;
	isStuck = false;
	reachedEndOfLevel = false;
}



bool Unit::checkIfTouchingBeam(std::shared_ptr<Beam> beam) {
	for (auto& beamSelected : listBeamsTouching)
		if (auto beamSelectedSP = beamSelected.lock())
			if (beamSelectedSP.get() == beam.get())
				return true;

	return false;
}



void Unit::addForceAtPin(Vector2D& forceSum) {
	//Determine how many pins the unit's force of gravity will be distributed over 
	//and output the total force divided by that amount.
	int pinCount = (int)listBeamsTouching.size() * 2;
	if (pinCount > 0) {
		float massAtPin = mass / pinCount;
		forceSum += Vector2D(0.0f, massAtPin * MathAddon::accGravity);
	}
}



bool Unit::getIsStuck() {
	return isStuck;
}


bool Unit::getReachedEndOfLevel() {
	return reachedEndOfLevel;
}



bool Unit::checkOverlapWithBridgeAndLand(Vector2D posCheck, 
	std::vector<std::shared_ptr<Beam>>& listBeams, bool isFirstIteration) {
	bool overlapFound = false;

	//If this is the first iteration then clear listBeamsTouching.
	if (isFirstIteration)
		listBeamsTouching.clear();

	//Loop through all the beams and see which ones overlap with this unit.
	for (auto& beamSelected : listBeams) {
		if (beamSelected != nullptr && beamSelected->checkOverlapWithUnit(posCheck, size / 2.0f)) {
			overlapFound = true;
			//If this is the first iteration then store the beam on listBeamsTouching.
			//This is done on the first iteration because that's when the possible new 
			//position being checked is going to overlap the most beams.
			if (isFirstIteration)
				listBeamsTouching.push_back(beamSelected);
		}
	}


	//Check overlap with land
	if (overlapFound == false) {
		//Left piece
		Vector2D posLeft1(0.0f, 387.0f);
		Vector2D posLeft2(264.0f, 387.0f);
		if (MathAddon::checkOverlapLineCircle(posLeft1, posLeft2, posCheck, (size / 2.0f + 11.0f)))
			overlapFound = true;

		if (overlapFound == false) {
			//Right piece
			Vector2D posRight1(696.0f, 387.0f);
			Vector2D posRight2(960.0f, 387.0f);
			if (MathAddon::checkOverlapLineCircle(posRight1, posRight2, posCheck, (size / 2.0f + 11.0f)))
				overlapFound = true;
		}
	}

	return overlapFound;
}